"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var get_type_1 = require("./get-type");
var TypeAssertions_1 = require("./TypeAssertions");
function normalizeField(field, fieldName, typeName, typeNames) {
    var ctx = typeName + '.' + fieldName;
    var f = TypeAssertions_1.default.String.or(TypeAssertions_1.default.ObjectKeys(['type', 'description', 'args', 'auth', 'resolve'])).validate(field, ctx);
    if (typeof f === 'string') {
        return {
            kind: "FieldProperty" /* FieldProperty */,
            name: typeName,
            description: undefined,
            resultType: get_type_1.default(f, ctx, typeNames),
            auth: 'public',
        };
    }
    var resolve = TypeAssertions_1.default.Void.or(TypeAssertions_1.default.Fn).validate(f.resolve, ctx + '.resolve');
    if (resolve) {
        return {
            kind: "FieldMethod" /* FieldMethod */,
            name: typeName,
            description: TypeAssertions_1.default.Void.or(TypeAssertions_1.default.String).validate(f.description, ctx + '.description'),
            resultType: get_type_1.default(f.type, ctx + '.type', typeNames),
            argType: get_type_1.default(f.args === undefined ? 'void' : f.args, ctx + '.args', typeNames),
            auth: TypeAssertions_1.default
                .Literal('public')
                .or(TypeAssertions_1.default.Fn)
                .validate(f.auth === undefined ? 'public' : f.auth, ctx + '.auth'),
            resolve: resolve,
        };
    }
    return {
        kind: "FieldProperty" /* FieldProperty */,
        name: typeName,
        description: TypeAssertions_1.default.Void.or(TypeAssertions_1.default.String).validate(f.description, ctx + '.description'),
        resultType: get_type_1.default(f.type, ctx + '.type', typeNames),
        auth: TypeAssertions_1.default
            .Literal('public')
            .or(TypeAssertions_1.default.Fn)
            .validate(f.auth === undefined ? 'public' : f.auth, ctx + '.auth'),
    };
}
exports.default = normalizeField;
//# sourceMappingURL=normalize-field.js.map