"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var assert = require("assert");
var get_type_1 = require("./get-type");
var TypeAssertions_1 = require("./TypeAssertions");
function normalizeScalar(Scalar, typeNames) {
    var s = TypeAssertions_1.default
        .ObjectKeys(['name', 'description', 'validate', 'baseType'])
        .validate(Scalar, 'Scalar');
    var name = TypeAssertions_1.default.String.validate(s.name, 'Scalar.name');
    assert(/^[A-Za-z]+$/.test(name), "Expected Scalar.name to match [A-Za-z]+ but got '" + name + "'");
    var description = TypeAssertions_1.default.Void.or(TypeAssertions_1.default.String).validate(s.description, name + '.description');
    var baseType = get_type_1.default(s.baseType, name + '.baseType', typeNames);
    var validate = TypeAssertions_1.default.Void.or(TypeAssertions_1.default.Fn).validate(s.validate, name + '.validate');
    return {
        kind: "Scalar" /* Scalar */,
        name: name,
        description: description,
        baseType: baseType,
        validate: (validate || (function () { return true; })),
    };
}
exports.default = normalizeScalar;
//# sourceMappingURL=normalize-scalar.js.map