"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
Object.defineProperty(exports, "__esModule", { value: true });
var run_query_against_cache_1 = require("./utils/run-query-against-cache");
var runner_1 = require("./runner");
var handleMessage_1 = require("./handleMessage");
var middleware_1 = require("./middleware");
var server_rendering_1 = require("./server-rendering");
var MemorySessionStore_1 = require("./sessions/MemorySessionStore");
var query_1 = require("./typed-helpers/query");
var Ctx_1 = require("./Ctx");
function noop() {
    return null;
}
var BicycleServer = /** @class */ (function () {
    function BicycleServer(schema, options) {
        if (options === void 0) { options = {}; }
        this._schema = schema;
        this._sessionStore =
            options.sessionStore || new MemorySessionStore_1.default(options.sessionStoreSize);
        this._logging = {
            disableDefaultLogging: options.disableDefaultLogging || false,
            onError: options.onError || noop,
            onRequestStart: options.onRequestStart || noop,
            onRequestEnd: options.onRequestEnd || noop,
            onMutationStart: options.onMutationStart || noop,
            onMutationEnd: options.onMutationEnd || noop,
            onQueryStart: options.onQueryStart || noop,
            onQueryEnd: options.onQueryEnd || noop,
        };
    }
    BicycleServer.prototype.runQuery = function (query, context) {
        var _this = this;
        var q = query instanceof query_1.BaseRootQuery ? query._query : query;
        return Ctx_1.default(context, function (context) {
            return runner_1.runQuery(q, {
                schema: _this._schema,
                logging: _this._logging,
                context: context,
            });
        }).then(function (cache) {
            var _a = run_query_against_cache_1.default(cache, q), loaded = _a.loaded, result = _a.result, errors = _a.errors;
            if (errors.length) {
                throw new Error(errors[0]);
            }
            if (!loaded) {
                throw new Error('Bicycle data failed to load, this should never happen.');
            }
            return result;
        });
    };
    BicycleServer.prototype.runMutation = function (method, args, context) {
        var _this = this;
        var m = method instanceof query_1.Mutation ? method._name : method;
        var a = method instanceof query_1.Mutation ? method._args : args;
        var c = method instanceof query_1.Mutation ? args : context;
        return Ctx_1.default(c, function (c) {
            return runner_1.runMutation({ method: m, args: a }, {
                schema: _this._schema,
                logging: _this._logging,
                context: c,
            });
        }).then(function (_a) {
            var s = _a.s, v = _a.v;
            if (s) {
                return v;
            }
            else {
                var err = new Error(v.message);
                err.data = v.data;
                err.code = v.code;
                throw err;
            }
        });
    };
    BicycleServer.prototype.handleMessage = function (message, getContext) {
        return __awaiter(this, void 0, void 0, function () {
            var result;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        this._logging.onRequestStart({ request: message });
                        return [4 /*yield*/, handleMessage_1.default(this._schema, this._logging, this._sessionStore, message, function () { return getContext({ stage: 'query' }); }, function () { return getContext({ stage: 'mutation' }); })];
                    case 1:
                        result = _a.sent();
                        this._logging.onRequestEnd({ request: message, response: result });
                        return [2 /*return*/, result];
                }
            });
        });
    };
    BicycleServer.prototype.createMiddleware = function (getContext) {
        return middleware_1.default(this._schema, this._logging, this._sessionStore, getContext);
    };
    BicycleServer.prototype.createServerRenderer = function (getContext, fn) {
        return server_rendering_1.default(this._schema, this._logging, this._sessionStore, getContext, fn);
    };
    BicycleServer.prototype.getNetworkLayer = function (getContext) {
        var _this = this;
        return { send: function (request) { return _this.handleMessage(request, getContext); } };
    };
    BicycleServer.prototype.getActiveSessionCount = function () {
        return this._sessionStore.getSessionCount
            ? this._sessionStore.getSessionCount()
            : null;
    };
    BicycleServer.prototype.getMaxSessionCount = function () {
        return this._sessionStore.getMaxSessionCount
            ? this._sessionStore.getMaxSessionCount()
            : null;
    };
    return BicycleServer;
}());
exports.default = BicycleServer;
//# sourceMappingURL=server-core.js.map